<?xml version="1.0"  encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
  	extension-element-prefixes="redirect">



<xsl:template name="standard-analysis-defined-metrics">
	<xsl:param name="defined-metrics-element"/>
	<xsl:param name="defined-metrics-elements-subcomponents"/>
	<xsl:param name="reports-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="package-element"/>
<!--  
	<xsl:message>
		standard-analysis-defined-metrics called with metrics:
		<xsl:for-each select="$defined-metrics-element/metric">
			metric:<xsl:value-of select="."/>
		</xsl:for-each>
	</xsl:message>
-->
    <h3>Scorecard</h3>
    <xsl:choose>
        <xsl:when test="$package-element">
            <p>
                For details on the package press the various categories in the submenu.
            </p>
        </xsl:when>
        <xsl:otherwise>
            <p>
                The graphs below gives the head defined metric in the system and shows how it has trended in the last versions. In the table
                on its side, it is shown what metric components it is made up of. Each of the subsequent graph and table is a identical view
                of that metric with the associated metric breakdown.
                <br/><br/>
                For metric details press the various categories in the submenu.
            </p>

            <table class="details">
                <xsl:for-each select="$defined-metrics-element/metric">
                    <xsl:call-template name="standard-dynamics-defined-metric-analysis-row">
                        <xsl:with-param name="reports-element" select="$reports-element"/>
                        <xsl:with-param name="metric-element" select="current()"/>
						<xsl:with-param name="defined-metrics-elements-subcomponents" select="$defined-metrics-elements-subcomponents"/>
                        <xsl:with-param name="name" select="$subsystem-name"/>
                     </xsl:call-template>

                     <xsl:for-each select="metric">
                         <tr><td style="background-color: white;" colspan="2"><hr size="1"/></td></tr>
                         <xsl:call-template name="standard-dynamics-defined-metric-analysis-row">
                            <xsl:with-param name="reports-element" select="$reports-element"/>
                            <xsl:with-param name="metric-element" select="current()"/>
							<xsl:with-param name="defined-metrics-elements-subcomponents" select="$defined-metrics-elements-subcomponents"/>
                            <xsl:with-param name="name" select="$subsystem-name"/>
                         </xsl:call-template>
                     </xsl:for-each>
                </xsl:for-each>
             </table>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template name="standard-dynamics-defined-metric-analysis-row">
    <xsl:param name="reports-element"/>
	<xsl:param name="metric-element"/>
    <xsl:param name="defined-metrics-elements-subcomponents"/>
	<xsl:param name="name"/>

    <xsl:variable
        name="image-name"
        select="concat('img/defined-metric-',@abbreviation,'-image-',$name,'.svg')"/>

     <redirect:write file="{$docs-home}/{$image-name}">
         <xsl:call-template name="standard-dynamics-defined-metric-line-graph">
            <xsl:with-param name="reports-element" select="$reports-element"/>
            <xsl:with-param name="metric-element" select="$metric-element"/>
			<xsl:with-param name="defined-metrics-elements-subcomponents" select="$defined-metrics-elements-subcomponents"/>
         </xsl:call-template>
    </redirect:write>

    <tr><td style="background-color: white;">
            <div align="center">
              <embed
                  width="500"
                  height="310"
                  src="{concat('../',$image-name)}"
                  name="svgmap"
                  pluginspage="http://www.adobe.com/svg/viewer/install/"
                  type="image/svg+xml"></embed>
            </div>
        </td>
        <td style="background-color: white;" valign="top">
            <xsl:call-template name="standard-dynamics-defined-metric-table">
                <xsl:with-param name="reports-element" select="$reports-element"/>
                <xsl:with-param name="metric-element" select="current()"/>
             </xsl:call-template>
        </td>
    </tr>

</xsl:template>




<xsl:template name="standard-dynamics-defined-metric-line-graph">
    <xsl:param name="reports-element"/>
	<xsl:param name="metric-element"/>
	<xsl:param name="defined-metrics-elements-subcomponents"/>
	<xsl:param name="y-axis-justification" select="'10-percent-normalize'"/>

	<xsl:variable name="metric-abbreviation" select="$metric-element/@abbreviation"/>
	<xsl:variable name="metric-name" select="$metric-element/@name"/>

	<xsl:call-template name="line-graph">
  		<xsl:with-param name="text-x-axis" select="'Version'"/>
  		<xsl:with-param name="text-y-axis" select="concat($metric-element/@name,' [',$metric-element/@abbreviation,']')"/>
  		<xsl:with-param name="width" select="'1300'"/>
  		<xsl:with-param name="height" select="'800'"/>
  		<xsl:with-param name="point-nodeset">
  			<points>
	  			<xsl:for-each select="$reports-element/report">
	  				<xsl:sort data-type="number" select="@seq-id"/>
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

                    <point>
                        <x-label><xsl:value-of select="version"/></x-label>
                        <x-coordinate><xsl:value-of select="sum(../report[@seq-id &lt;= $current-seq-id]/days-since-last)"/></x-coordinate>
                        <y-coordinate><xsl:value-of select="$metric-element/value[@seq-id = $current-seq-id]/@value"/></y-coordinate>
                    </point>
                </xsl:for-each>
  			</points>
			<xsl:if test="$defined-metrics-elements-subcomponents">
			<subcomponents>
				<xsl:for-each select="$defined-metrics-elements-subcomponents/subsystem">
				<component name="{@name}">
					<xsl:variable name="this-subsystem" select="current()"/>
					<points>
						  <xsl:for-each select="$reports-element/report">
							  <xsl:sort data-type="number" select="@seq-id"/>
							  <xsl:variable name="current-seq-id" select="@seq-id"/>
								<point>
									<x-label><xsl:value-of select="version"/></x-label>
									<x-coordinate><xsl:value-of select="sum(../report[@seq-id &lt;= $current-seq-id]/days-since-last)"/></x-coordinate>
									<y-coordinate><xsl:value-of select="$this-subsystem/descendant-or-self::metric[@abbreviation = $metric-abbreviation]/value[@seq-id = $current-seq-id]/@value"/></y-coordinate>
								</point>
						</xsl:for-each>
					</points>
				</component>
				</xsl:for-each>
			</subcomponents>
			</xsl:if>
  		</xsl:with-param>
  		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
	</xsl:call-template>

</xsl:template>

<xsl:template name="standard-dynamics-defined-metric-table">
    <xsl:param name="reports-element"/>
	<xsl:param name="metric-element"/>

	<table class="details">
        <tr>
            <th>Version</th>
            <th>Date</th>
            <th><xsl:value-of select="concat($metric-element/@name,' [',$metric-element/@abbreviation,']')"/></th>
            <xsl:for-each select="$metric-element/metric">
            <th><xsl:value-of select="concat(@name,' [',@abbreviation,']')"/></th>
            </xsl:for-each>
        </tr>

        <xsl:for-each select="$reports-element/report">
            <xsl:sort data-type="number" select="@seq-id"/>
            <xsl:variable name="current-seq-id" select="@seq-id"/>

            <tr>
                <td><xsl:value-of select="version"/></td>
                <td><xsl:value-of select="date"/></td>
                <td><xsl:value-of select="format-number($metric-element/value[@seq-id = $current-seq-id]/@value,'#.##')"/></td>
                <xsl:for-each select="$metric-element/metric">
                    <td><xsl:value-of select="format-number(value[@seq-id = $current-seq-id]/@value,'#.##')"/></td>
                </xsl:for-each>
            </tr>
        </xsl:for-each>
    </table>

</xsl:template>

</xsl:stylesheet>
