<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:lxslt="http://xml.apache.org/xslt"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
    extension-element-prefixes="redirect">

<xsl:template match="IntegratedAnalysis" mode="scorecard-package-design">

	<xsl:variable name="packages-element" select="Packages"/>

   	<html>
      <head>
         <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>

         <xsl:call-template name="page-header">
  	  	   <xsl:with-param name="main-menu-page-name" select="'scorecard'"/>
  	  	   <xsl:with-param name="page-name" select="'package design'"/>
           <xsl:with-param name="dynamics-page-name" select="'package-design'"/>
  	  	   <xsl:with-param name="root-element" select="current()"/>
  	     </xsl:call-template>

		<h3>Standard OO Metrics</h3>
			Below you see the standard Object oriented metrics in tabular form. According to a NASA Study, they found the following values.
		<p/>
		  	<table class="details">
				<tr><th>System analyzed</th><th>Java</th><th>Java</th><th>C++</th></tr>
				<tr><td>Classes</td><td>46</td><td>1000</td><td>1617</td></tr>
				<tr><td>Lines</td><td>50,000</td><td>300,000</td><td>500,000</td></tr>
		  	</table>
      	<p>

      	</p>
	      	<table class="details">
    	  		<tr><td>Quality</td><td>"Low"</td><td>"High"</td><td>"Medium"</td></tr>
				<tr><td>CBO</td><td>2.48</td><td>1.25</td><td>2.09</td></tr>
				<tr><td>LCOM</td><td>447.65</td><td>78.34</td><td>113.94</td></tr>
				<tr><td>RFC</td><td>80.39</td><td>43.84</td><td>28.60</td></tr>
				<tr><td>NOC</td><td>0.07</td><td>0.35</td><td>0.39</td></tr>
				<tr><td>DIT</td><td>0.37</td><td>0.97</td><td>1.02</td></tr>
				<tr><td>WMC</td><td>45.7</td><td>11.10</td><td>23.97</td></tr>
			</table>
		<p/>
		  The figures suggest that the higher the CBO and WMC, the lower the quality of the system. This seems to hold for LCOM as well.
		<p/>
		<table width="100%" class="details">
			<tr>
	            <td style="background-color: white;">
					<table width="100%" class="details">
					    <xsl:call-template name="metric-header-package-design"/>

						<xsl:variable name="packages" select="count($packages-element/Package)"/>
					    <xsl:call-template name="subsystem-design-metrics">
							<xsl:with-param name="subsystem-name" select="'Total'"/>
							<xsl:with-param name="subsystem-link" select="'none'"/>
							<xsl:with-param name="total-packages" select="count($packages-element/Package/Stats)"/>
							<xsl:with-param name="distance" select="sum($packages-element/Package/Stats/D) div $packages"/>
							<xsl:with-param name="efferent-couplings" select="sum($packages-element/Package/Stats/Ce)"/>
							<xsl:with-param name="afferent-couplings" select="sum($packages-element/Package/Stats/Ca)"/>
							<xsl:with-param name="packages-with-cycles" select="count($packages-element/Package/Cycles)"/>
							<xsl:with-param name="illegal-subsystem-dependencies" select="sum($packages-element/Package/Stats/dependancy-metrics/subsystem[@legal='false'])"/>
							<xsl:with-param name="legal-subsystem-dependencies" select="sum($packages-element/Package/Stats/dependancy-metrics/subsystem[@legal='true'])"/>
							<xsl:with-param name="wmc" select="sum($packages-element/Package/Stats/oo-metrics/wmc) div $packages"/>
							<xsl:with-param name="dit" select="sum($packages-element/Package/Stats/oo-metrics/dit) div $packages"/>
							<xsl:with-param name="noc" select="sum($packages-element/Package/Stats/oo-metrics/noc) div $packages"/>
							<xsl:with-param name="cbo" select="sum($packages-element/Package/Stats/oo-metrics/cbo) div $packages"/>
							<xsl:with-param name="ca" select="sum($packages-element/Package/Stats/oo-metrics/ca) div $packages"/>
							<xsl:with-param name="rfc" select="sum($packages-element/Package/Stats/oo-metrics/rfc) div $packages"/>
							<xsl:with-param name="lcom" select="sum($packages-element/Package/Stats/oo-metrics/lcom) div $packages"/>
						</xsl:call-template>

						<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
							<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
							<xsl:variable name="file-name"><xsl:call-template name="subsystem-url">
								<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
								<xsl:with-param name="anchor" select="'design-metrics'"/>
							</xsl:call-template></xsl:variable>
							<xsl:variable name="total-packages" select="count($packages-element/Package[subsystem/@name = $subsystem-name]/Stats)"/>
							<xsl:call-template name="subsystem-design-metrics">
								<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
								<xsl:with-param name="subsystem-link" select="$file-name"/>
								<xsl:with-param name="total-packages" select="$total-packages"/>
								<xsl:with-param name="distance" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/D) div $total-packages"/>
								<xsl:with-param name="efferent-couplings" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/Ce)"/>
								<xsl:with-param name="afferent-couplings" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/Ca)"/>
								<xsl:with-param name="packages-with-cycles" select="count($packages-element/Package[subsystem/@name = $subsystem-name]/Cycles)"/>
								<xsl:with-param name="illegal-subsystem-dependencies" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/dependancy-metrics/subsystem[@legal='false'])"/>
								<xsl:with-param name="legal-subsystem-dependencies" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/dependancy-metrics/subsystem[@legal='true'])"/>
								<xsl:with-param name="wmc" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/wmc) div $total-packages"/>
								<xsl:with-param name="dit" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/dit) div $total-packages"/>
								<xsl:with-param name="noc" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/noc) div $total-packages"/>
								<xsl:with-param name="cbo" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/cbo) div $total-packages"/>
								<xsl:with-param name="ca" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/ca) div $total-packages"/>
								<xsl:with-param name="rfc" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/rfc) div $total-packages"/>
								<xsl:with-param name="lcom" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/oo-metrics/lcom) div $total-packages"/>
							</xsl:call-template>
						</xsl:for-each>
					</table>
				</td>
			</tr>
		</table>


		<h3>Instability-Abstractness</h3>
		The packages should be placed along the
		main sequence (marked with green in the figure). The top right corner is often called "the zone of uselessness" due to the
		package contents high abstractness and low usage. The opposite corner (bottom right) has been named "the zone
		of pain" due to low degree of abstraction and high usage. Changes in these packages usually imply extensive changes in
		the rest of the system.
		<p/>
		Roll over the figure with your mouse to see the statistics. Press the mouse button to see the details of that package.
		<p/>
		<table width="100%" class="details">
		    <tr>
	            <td style="background-color: white;">
			 		<div align="top" valign="center">
		              <embed width="650" height="650" src="img/abstractness-instability-graph.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
			</tr>
		</table>

				<h3>Public API</h3>

		<table width="100%" class="details">


		    <xsl:call-template name="metric-header-system-api"/>

			<xsl:call-template name="subsystem-system-api-metrics">
				<xsl:with-param name="subsystem-name" select="'Total'"/>
				<xsl:with-param name="subsystem-link" select="'none'"/>
				<xsl:with-param name="api-packages" select="count($packages-element/Package[@public-api-package='true'])"/>
				<xsl:with-param name="api-classes" select="sum($packages-element/Package[@public-api-package='true']/Stats/TotalClasses)"/>
				<xsl:with-param name="api-methods" select="sum($packages-element/Package[@public-api-package='true']/Stats/functions)"/>
                <xsl:with-param name="api-source-statements" select="sum($packages-element/Package[@public-api-package='true']/Stats/ncss)"/>
				<xsl:with-param name="api-import-couplings" select="sum($packages-element/Package[@public-api-package='true']/Stats/coupling-metrics/import-coupling)"/>
				<xsl:with-param name="api-export-couplings" select="sum($packages-element/Package[@public-api-package='true']/Stats/coupling-metrics/export-coupling)"/>
				<xsl:with-param name="packages" select="count($packages-element/Package)"/>
				<xsl:with-param name="classes" select="sum($packages-element/Package/Stats/TotalClasses)"/>
				<xsl:with-param name="methods" select="sum($packages-element/Package/Stats/functions)"/>
                <xsl:with-param name="source-statements" select="sum($packages-element/Package/Stats/ncss)"/>
				<xsl:with-param name="import-couplings" select="sum($packages-element/Package/Stats/coupling-metrics/import-coupling)"/>
				<xsl:with-param name="export-couplings" select="sum($packages-element/Package/Stats/coupling-metrics/export-coupling)"/>
			</xsl:call-template>
			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
				<xsl:variable name="file-name"><xsl:call-template name="subsystem-url">
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="anchor" select="'system-api'"/>
                </xsl:call-template></xsl:variable>
				<xsl:call-template name="subsystem-system-api-metrics">
					<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="subsystem-link" select="$file-name"/>
					<xsl:with-param name="api-packages" select="count($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')])"/>
					<xsl:with-param name="api-classes" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')]/Stats/TotalClasses)"/>
					<xsl:with-param name="api-methods" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')]/Stats/functions)"/>
					<xsl:with-param name="api-source-statements" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')]/Stats/ncss)"/>
					<xsl:with-param name="api-import-couplings" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')]/Stats/coupling-metrics/import-coupling)"/>
					<xsl:with-param name="api-export-couplings" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name) and (@public-api-package='true')]/Stats/coupling-metrics/export-coupling)"/>
					<xsl:with-param name="packages" select="count($packages-element/Package[(subsystem/@name = $subsystem-name)])"/>
					<xsl:with-param name="classes" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name)]/Stats/TotalClasses)"/>
					<xsl:with-param name="methods" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name)]/Stats/functions)"/>
					<xsl:with-param name="source-statements" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name)]/Stats/ncss)"/>
					<xsl:with-param name="import-couplings" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name)]/Stats/coupling-metrics/import-coupling)"/>
					<xsl:with-param name="export-couplings" select="sum($packages-element/Package[(subsystem/@name = $subsystem-name)]/Stats/coupling-metrics/export-coupling)"/>
				</xsl:call-template>
			</xsl:for-each>
		</table>

      </body>
   </html>
</xsl:template>

<xsl:template name="metric-header-package-design">
	<xsl:param name="single-package"/>

	<tr>
       <th>Package Design</th>
		<xsl:if test="not($single-package='true')">
		<th>Total Packages</th>
		</xsl:if>
		<th>Distance</th>
		<th>Efferent Couplings</th>
		<th>Afferent Couplings</th>
		<th>Cycles</th>
		<th>Illegal dep</th>
		<th>Legal dep</th>
		<th>Methods per Class [WMC]</th>
		<th>Depth of inheritance tree [DIT]</th>
		<th>Number of Children [NOC]</th>
		<th>Efferent Class Couplings [CBO/Ce]</th>
		<th>Afferent Class Couplings [Ca]</th>
		<th>Response for a Class [RFC]</th>
		<th>Lack of Cohesion [LCOM]</th>
    </tr>
</xsl:template>

<xsl:template name="subsystem-design-metrics">
	<xsl:param name="subsystem-name"/>
	<xsl:param name="subsystem-link"/>
	<xsl:param name="total-packages"/>
	<xsl:param name="distance"/>
	<xsl:param name="efferent-couplings"/>
	<xsl:param name="afferent-couplings"/>
	<xsl:param name="packages-with-cycles"/>
	<xsl:param name="single-package"/>
	<xsl:param name="illegal-subsystem-dependencies"/>
	<xsl:param name="legal-subsystem-dependencies"/>
	<xsl:param name="wmc" select="''"/>
	<xsl:param name="dit" select="''"/>
	<xsl:param name="noc" select="''"/>
	<xsl:param name="cbo" select="''"/>
	<xsl:param name="ca" select="''"/>
	<xsl:param name="rfc" select="''"/>
	<xsl:param name="lcom" select="''"/>

	<tr>
	 <td align="left">
	 	<xsl:choose>
	 		<xsl:when test="$subsystem-link != 'none'">
			    <a href="{$subsystem-link}">
			       <xsl:value-of select="$subsystem-name"/>
			    </a>
			</xsl:when>
			<xsl:otherwise>
				<b><xsl:value-of select="$subsystem-name"/></b>
			</xsl:otherwise>
	    </xsl:choose>
	 </td>

	 <xsl:if test="not($single-package = 'true')">
	 	<td align="right"><xsl:value-of select="$total-packages"/></td>
	 </xsl:if>

	 <td align="right"><xsl:value-of select="format-number($distance, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($efferent-couplings, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($afferent-couplings, '#.##')"/></td>
	 <xsl:choose>
	 	<xsl:when test="$single-package='true'">
	 		<td align="center">
		 		<xsl:if test="$packages-with-cycles &gt; 0">
		 			X
		 		</xsl:if>
	 		</td>
	 	</xsl:when>
	 	<xsl:otherwise>
	 		<td align="right"><xsl:value-of select="format-number(100 * $packages-with-cycles div $total-packages, '#.##')"/>%</td>
	 	</xsl:otherwise>
	 </xsl:choose>
	 <td align="right"><xsl:value-of select="$illegal-subsystem-dependencies"/></td>
	 <td align="right"><xsl:value-of select="$legal-subsystem-dependencies"/></td>
	 <td align="right"><xsl:value-of select="format-number($wmc, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($dit, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($noc, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($cbo, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($ca, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($rfc, '#.##')"/></td>
	 <td align="right"><xsl:value-of select="format-number($lcom, '#.##')"/></td>

	</tr>
</xsl:template>


<xsl:template name="metric-header-system-api">

	<tr>
       <th>System API</th>
		<th>Total API Packages</th>
		<th>Total API Classes</th>
		<th>Total API Methods</th>
		<th>Source Statements</th>
		<th>Import Couplings</th>
		<th>Export Couplings</th>

    </tr>
</xsl:template>

<xsl:template name="subsystem-system-api-metrics">
	<xsl:param name="subsystem-name"/>
	<xsl:param name="subsystem-link"/>
    <xsl:param name="api-packages"/>
	<xsl:param name="api-classes"/>
	<xsl:param name="api-methods"/>
	<xsl:param name="api-source-statements"/>
	<xsl:param name="api-import-couplings"/>
	<xsl:param name="api-export-couplings"/>
	<xsl:param name="packages"/>
	<xsl:param name="classes"/>
	<xsl:param name="methods"/>
	<xsl:param name="source-statements"/>
	<xsl:param name="import-couplings"/>
	<xsl:param name="export-couplings"/>

	<tr>
        <td align="left">
	 	<xsl:choose>
	 		<xsl:when test="$subsystem-link != 'none'">
			    <a href="{$subsystem-link}">
			       <xsl:value-of select="$subsystem-name"/>
			    </a>
			</xsl:when>
			<xsl:otherwise>
				<b><xsl:value-of select="$subsystem-name"/></b>
			</xsl:otherwise>
	    </xsl:choose>
	 </td>
	   	<td align="right"><xsl:value-of select="$api-packages"/><xsl:if test="($api-packages &gt; 0) and ($packages &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-packages div $packages,'#.#')"/>%)</xsl:if></td>
       	<td align="right"><xsl:value-of select="$api-classes"/><xsl:if test="($api-classes &gt; 0) and ($classes &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-classes div $classes,'#.#')"/>%)</xsl:if></td>
       	<td align="right"><xsl:value-of select="$api-methods"/><xsl:if test="($api-methods &gt; 0) and ($methods &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-methods div $methods,'#.#')"/>%)</xsl:if></td>
       	<td align="right"><xsl:value-of select="$api-source-statements"/><xsl:if test="($api-source-statements &gt; 0) and ($source-statements &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-source-statements div $source-statements,'#.#')"/>%)</xsl:if></td>
       	<td align="right"><xsl:value-of select="$api-import-couplings"/><xsl:if test="($api-import-couplings &gt; 0) and ($import-couplings &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-import-couplings div $import-couplings,'#.#')"/>%)</xsl:if></td>
       	<td align="right"><xsl:value-of select="$api-export-couplings"/><xsl:if test="($api-export-couplings &gt; 0) and ($export-couplings &gt; 0)"> (<xsl:value-of select="format-number(100 * $api-export-couplings div $export-couplings,'#.#')"/>%)</xsl:if></td>
	</tr>
</xsl:template>

</xsl:stylesheet>
